(function($) {

  $(document).ready( function() {
    $.fn.quicktext.init();
  });

  $.fn.quicktext = {

    init: function(options) {
      var $settings = Drupal.settings.quicktext;
      if ($($settings).length <= 0) return false;

      var settings = {
        'show': $settings.quicktext_show,
        'buttons': $settings.quicktext_buttons,
        'formats': $settings.quicktext_input_formats
      };

      if (options) { $.extend(settings, options); }

      // Will do input formats crosschecking here later on...
      switch (settings.show) {
        case '1':
          var $textareas = $('#node-form .form-item textarea');
          break;
        case '2':
          var $textareas = $('#node-form .form-item textarea, #comment-form .form-item textarea');
          break;
        case '3':
          var $textareas = $('.form-item textarea');
          break;
        default:
          return false;
      }

      if ($textareas.length == 0) return false;

      $textareas.each( function() {
        var $textarea = $(this);
        var $buttons = $('<div class="quicktext_container"></div>');
        var $clear = $('<div class="quicktext_clear">&nbsp;</div>'); // My clear forever until...

        $textarea.addClass('resizable');
        $textarea.before($buttons);
        $.each(settings.buttons, function(i, button) {
          var $button = $('<div class="quicktext_button"></div>');

          if (button.src) {
            var $btn_img = $('<img class="' + i + '"></img>');
            $btn_img.attr('src', Drupal.settings.basePath + button.src);
          } else {
            var btn_title = (button.name) ? button.name : button.title;
            var $btn_img = $('<a href="#" class="btn ' + i + '"><span><span><b>&nbsp;</b><u>' + btn_title + '</u></span></span></a>');
          }

          $btn_img.attr('title', button.title).click( function(e) { e.preventDefault(); } );

          $button.click( function(e) {
            e.preventDefault();
            var content = $textarea.attr('value');
            var len = content.length;
            var start = $textarea.get(0).selectionStart;
            var end = $textarea.get(0).selectionEnd;
            var sel = content.substring(start, end);

            if (button.suffix) {
              if (i == 'ed_unorderedlist' || i == 'ed_orderedlist') {
                var lines = sel.split(/\n/g);
                if ($(lines).length > 0) {
                  var replace = button.prefix + "\n" + "  <li>" + lines.join("</li>" + "\n" + "  <li>") + "</li>" + "\n" +button.suffix;
                } else {
                  return false;
                }
              } else {
                var replace = button.prefix + sel + button.suffix;
              }
            }
            else if (button.prefix) {
              var replace = button.prefix + sel;
            } else if (i == 'quicktext_view') {
              var $current_preview = $('.quicktext_preview');

              if ($current_preview.length == 0) {
                var $preview = $('<div class="quicktext_preview"></div>');
                var $content = $('<div class="content"></div>');

                $preview.append($content);
                $buttons.append($preview);

                var preview_width = $buttons.width();
                var preview_height = $textarea.innerHeight() + 100;

                $preview.animate({
                  width: preview_width,
                  height: preview_height,
                  opacity: 1          
                }, 'normal');

                var pb = new Drupal.progressBar('quicktext-preview-progress');
                var $pb = $(pb.element);
                var $form = $textarea.parents('form').eq(0);
                var token = $form.find('input[name=form_token]').attr('value');
                var token_value = $form.find('input[name=quicktext_token]').attr('value');
                var format = $form.find('input[name=format][type=radio][checked]').attr('value');

                $preview.append($pb).fadeIn('slow');

                $.post(Drupal.settings.basePath + 'quicktext_preview', {
                  'token': token,
                  'token_value': token_value,
                  'format': format,
                  'is_node_form': ($textarea.attr('id') == 'edit-body') ? 1 : 0,
                  'content': content
                }, function(content_preview) {
                  $content.append(content_preview);
                  $pb.fadeOut('fast', function() {
                    $(this).remove();
                  });
                });
              }
              else {
                $current_preview.animate({
                  width: 0,
                  height: 0,
                  opacity: 0          
                }, 'normal', 'linear', function() {
                  $(this).remove();
                });
              }
            }

            if (button.prefix || button.suffix) {
              $textarea.attr('value', content.substring(0, start) + replace + content.substring(end, len));
            }

            if (i == 'quicktext_link') {
              $textarea.focus();
              $textarea.quicktext_setCursorPosition((start + button.seloffset));
            }
            else if (i == 'quicktext_ul' || i == 'quicktext_ol') {
              $textarea.focus(); // @todo
              $textarea.quicktext_setCursorPosition((end + (button.suffix.length - 1)));
            }
            else if (end && sel && button.suffix) {
              $textarea.focus();
              $textarea.quicktext_setCursorPosition((end + (button.suffix.length - 1)));
            }
            else if (i != 'quicktext_view') {
              $textarea.focus();
            }
          });
          if (button.qt_class) {
            $button.addClass(button.qt_class);
          }
          if ((i != 'quicktext_break' && i != 'ed_comment' && i != 'ed_heading3') || $textarea.attr('id') == 'edit-body') {
            $button.append($btn_img);
            $buttons.append($button);
          }
          if (i == 'ed_comment') {
            $buttons.append($clear.clone());
          }
        });
        $buttons.append($clear);
      });

      return $textareas;
    }

  };

  $.fn.quicktext_setCursorPosition = function(pos) {
    if ($(this).get(0).setSelectionRange) {
      $(this).get(0).setSelectionRange(pos, pos);
    } else if ($(this).get(0).createTextRange) {
      var range = $(this).get(0).createTextRange();
      range.collapse(true);
      range.moveEnd('character', pos);
      range.moveStart('character', pos);
      range.select();
    }
  };

})(jQuery);
